@@ -10,7 +10,7 @@ from paramiko import SFTPClient
 
 from hypernets.hyperctl import consts
 from hypernets.hyperctl.utils import http_portal
-from hypernets.hyperctl.batch import ShellJob
+from hypernets.hyperctl.batch import _ShellJob
 from hypernets.utils import logging as hyn_logging
 from hypernets.utils import ssh_utils
 import shutil
@@ -25,12 +25,12 @@ class NoResourceException(Exception):
 
 class ShellExecutor:
 
-    def __init__(self, job: ShellJob, api_server_portal, environments=None):
+    def __init__(self, job: _ShellJob, api_server_portal, environments=None):
         self.job = job
         self.api_server_portal = api_server_portal
         self.environments = environments
 
-    def run(self):
+    def run(self, *, independent_tmp=True):
         pass
 
     def post(self):
@@ -45,38 +45,43 @@ class ShellExecutor:
     def status(self):
         raise NotImplemented
 
-    def _make_run_shell_content(self):
+    def _make_run_shell_content(self, independent_tmp):
 
-        # default http://localhost:8060
-        vars = {
-            consts.KEY_ENV_JOB_DATA_DIR: self.job.job_data_dir,
+        template_vars = {
+            consts.KEY_ENV_JOB_DATA_DIR: self.job.data_dir_path.as_posix(),
             consts.KEY_ENV_JOB_NAME: self.job.name,
             consts.KEY_ENV_JOB_WORKING_DIR: self.job.working_dir,  # default value
-            consts.KEY_TEMPLATE_COMMAND: self.job.command,
-            consts.KEY_ENV_SERVER_PORTAL: self.api_server_portal,
+            consts.KEY_TEMPLATE_COMMAND: self.job.batch.job_command,
+            consts.KEY_ENV_SERVER_PORTAL: self.api_server_portal
         }
 
         run_shell = str(consts.RUN_SH_TEMPLATE)
-        for k, v in vars.items():
+        for k, v in template_vars.items():
             run_shell = run_shell.replace(f"#{k}#", v)
 
         export_custom_envs_command = ""
         if self.environments is not None:
             for k, v in self.environments.items():
-                export_command = f"export {k}=\"{v}\"\n"
+                export_command = f"export {k}={v}\n"
                 export_custom_envs_command = export_custom_envs_command + export_command
 
-        run_shell = run_shell.replace(f"#{consts.P_HOST_ENV}", export_custom_envs_command)
+        export_tmp_command = ""
+        if independent_tmp:
+            independent_tmp_path = (Path(tempfile.gettempdir()) / consts.BATCH_TEMP / self.job.name).as_posix()
+            export_tmp_command = f"export TMP={independent_tmp_path}"
+
+        run_shell = run_shell.replace(f"#{consts.P_HOST_ENV}", export_custom_envs_command)\
+            .replace(f"#{consts.P_TMP_ENV}", export_tmp_command)
 
         return run_shell
 
-    def _write_run_shell_script(self, file_path):
+    def _write_run_shell_script(self, file_path, independent_tmp):
         file_path = Path(file_path)
         # check parent
         if not file_path.parent.exists():
             os.makedirs(Path(file_path.parent), exist_ok=True)
         # write file
-        shell_content = self._make_run_shell_content()
+        shell_content = self._make_run_shell_content(independent_tmp)
         logger.debug(f'write shell script for job {self.job.name} to {file_path} ')
         with open(file_path, 'w', newline='\n') as f:
             f.write(shell_content)
@@ -106,9 +111,9 @@ class LocalShellExecutor(ShellExecutor):
             else:
                 shutil.copyfile(asset_file, (self.job.resources_path / asset_path.name).as_posix())
 
-    def run(self):
+    def run(self, *, independent_tmp=True):
         # prepare data dir
-        job_data_dir = Path(self.job.job_data_dir)
+        job_data_dir = Path(self.job.data_dir_path)
 
         if not job_data_dir.exists():
             os.makedirs(job_data_dir, exist_ok=True)
@@ -116,22 +121,22 @@ class LocalShellExecutor(ShellExecutor):
         self.prepare_assets()
 
         # create shell file & write to local
-        self._write_run_shell_script(self.job.run_file_path)
+        self._write_run_shell_script(self.job.run_file, independent_tmp)
 
-        command = f'sh {self.job.run_file_path}'
+        command = f'sh {self.job.run_file}'
 
         self._process = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE)
         logger.debug(f'executed command {command} , pid is {self._process.pid}')
 
     def status(self):
         if self._process is None:
-            return ShellJob.STATUS_INIT
+            return _ShellJob.STATUS_INIT
         process_ret_code = self._process.poll()
         if process_ret_code is None:
-            return ShellJob.STATUS_RUNNING
+            return _ShellJob.STATUS_RUNNING
         if process_ret_code == 0:
-            return ShellJob.STATUS_SUCCEED
-        return ShellJob.STATUS_FAILED
+            return _ShellJob.STATUS_SUCCEED
+        return _ShellJob.STATUS_FAILED
 
     def kill(self):
         if self._process is not None:
@@ -176,10 +181,17 @@ class SSHRemoteMachine:
             return False
 
     def test_connection(self):
-        # check connections
-        with ssh_utils.ssh_client(**self.connection) as client:
+        client = None
+        try:
+            # check connections
+            client = ssh_utils.create_ssh_client(**self.connection)
             assert client
-            logger.info(f"test connection to host {self.hostname} successfully")
+            logger.info(f"connect to host {self.hostname} successfully")
+        except Exception as e:
+            logger.exception("test connection failed")
+        finally:
+            if client is not None:
+                client.close()
 
     @staticmethod
     def total_resources():
@@ -196,8 +208,9 @@ class SSHRemoteMachine:
     def hostname(self):
         return self.connection['hostname']
 
+
 class RemoteShellExecutor(ShellExecutor):
-    def __init__(self, job: ShellJob, api_server_portal, machine: SSHRemoteMachine):
+    def __init__(self, job: _ShellJob, api_server_portal, machine: SSHRemoteMachine):
         super(RemoteShellExecutor, self).__init__(job, api_server_portal, environments=machine.environments)
         self.machine = machine
 
@@ -208,62 +221,80 @@ class RemoteShellExecutor(ShellExecutor):
     def connections(self):
         return self.machine.connection
 
-    def prepare_assets(self):
+    def prepare_assets(self, sftp_client):
         if len(self.job.assets) == 0:
             return
-        with ssh_utils.sftp_client(**self.connections) as sftp_client:
-            for asset in self.job.assets:
-                asset_path = Path(asset).absolute()
-                asset_file = asset_path.as_posix()
-                if not asset_path.exists():
-                    logger.warning(f"local dir {asset_path} not exists, skip to upload")
-                    continue
-                if asset_path.is_dir():
-                    ssh_utils.upload_dir(sftp_client, asset_file, self.job.resources_path.as_posix())
-                else:
-                    ssh_utils.upload_file(sftp_client, asset_file, (self.job.resources_path / asset_path.name).as_posix())
 
-    def run(self):
-        output_dir = Path(self.job.output_dir).as_posix()
-        logger.debug(f"create remote data dir {output_dir}")
-        with ssh_utils.sftp_client(**self.connections) as sftp_client:
-            logger.debug(f"create remote job output dir {output_dir} ")
-            ssh_utils.makedirs(sftp_client, output_dir)
+        for asset in self.job.assets:
+            asset_path = Path(asset).absolute()
+            asset_file = asset_path.as_posix()
+            if not asset_path.exists():
+                logger.warning(f"local dir {asset_path} not exists, skip to upload")
+                continue
+            if asset_path.is_dir():
+                ssh_utils.upload_dir(sftp_client, asset_file, self.job.resources_path.as_posix())
+            else:
+                ssh_utils.upload_file(sftp_client, asset_file, (self.job.resources_path / asset_path.name).as_posix())
 
-        logger.debug(f"prepare to upload assets to {self.machine.hostname}")
-        self.prepare_assets()
+    def run(self, *,  independent_tmp=True):
+        data_dir = self.job.data_dir_path.as_posix()
+        logger.debug(f"create remote data dir {data_dir}")
+        ssh_conn = None
+        sftp_client = None
+        try:
+            ssh_conn = ssh_utils.create_ssh_client(**self.connections)
 
-        # create run shell file
-        fd_run_file, run_file = tempfile.mkstemp(prefix=f'hyperctl_run_{self.job.name}_', suffix='.sh')
-        os.close(fd_run_file)
+            sftp_client = ssh_conn.open_sftp()
+            # with ssh_utils.sftp_client(**self.connections) as sftp_client:
+            logger.debug(f"create remote job output dir {data_dir} ")
+            ssh_utils.makedirs(sftp_client, data_dir)
 
-        self._write_run_shell_script(run_file)
+            logger.debug(f"prepare to upload assets to {self.machine.hostname}")
+            self.prepare_assets(sftp_client)  # share connection
 
-        # copy file to remote
-        with ssh_utils.sftp_client(**self.connections) as sftp_client:
-            logger.debug(f'upload {run_file} to {self.job.run_file_path}')
-            sftp_client: SFTPClient = sftp_client
-            ssh_utils.upload_file(sftp_client, run_file, self.job.run_file_path)
+            # create run sh file
+            fd_run_file, run_file = tempfile.mkstemp(prefix=f'hyperctl_run_{self.job.name}_', suffix='.sh')
+            os.close(fd_run_file)
 
-        # execute command in async
-        self._command_ssh_client = ssh_utils.create_ssh_client(**self.connections)
-        command = f'sh {self.job.run_file_path}'
-        logger.debug(f'execute command {command}')
-        self._remote_process = self._command_ssh_client.exec_command(command, get_pty=True)
+            self._write_run_shell_script(run_file, independent_tmp)
+
+            # copy file to remote
+            # with ssh_utils.sftp_client(**self.connections) as sftp_client:
+            logger.debug(f'upload {run_file} to {self.job.run_file}')
+            ssh_utils.upload_file(sftp_client, run_file, self.job.run_file)
+
+            if sftp_client is not None:
+                sftp_client.close()
+
+            # execute command in async
+            # self._command_ssh_client = ssh_utils.create_ssh_client(**self.connections)
+            self._command_ssh_client = ssh_conn
+
+            command = f'sh {self.job.run_file}'
+            logger.debug(f'execute command {command}')
+            self._remote_process = self._command_ssh_client.exec_command(command, get_pty=True)
+        except Exception as e:
+            if ssh_conn is not None:
+                ssh_conn.close()
+            if sftp_client is not None:
+                sftp_client.close()
+            raise e
+        finally:
+            pass
 
     def status(self):
         if self._remote_process is not None:
             stdout = self._remote_process[1]
             if stdout.channel.exit_status_ready():
                 ret_code = stdout.channel.recv_exit_status()
-                return ShellJob.STATUS_SUCCEED if ret_code == 0 else ShellJob.STATUS_FAILED
+                return _ShellJob.STATUS_SUCCEED if ret_code == 0 else _ShellJob.STATUS_FAILED
             else:
-                return ShellJob.STATUS_RUNNING
+                return _ShellJob.STATUS_RUNNING
         else:
-            return ShellJob.STATUS_INIT
+            return _ShellJob.STATUS_INIT
 
     def finished(self):
-        return self.status() in ShellJob.FINAL_STATUS
+        return self.status() in _ShellJob.FINAL_STATUS
 
     def kill(self):
         self.close()
@@ -279,6 +310,7 @@ class ExecutorManager:
 
     def __init__(self, api_server_portal):
         self.api_server_portal = api_server_portal
+
         self._waiting_queue = []
 
     def allocated_executors(self):
@@ -313,8 +345,8 @@ class RemoteSSHExecutorManager(ExecutorManager):
 
     def __init__(self, api_server_portal, machines: List[SSHRemoteMachine]):
         super(RemoteSSHExecutorManager, self).__init__(api_server_portal)
-
         self.machines = machines
+
         self._executors_map = {}
 
     def allocated_executors(self):
@@ -325,11 +357,18 @@ class RemoteSSHExecutorManager(ExecutorManager):
             if machine.usage == (0, 0, 0):
                 ret = machine.alloc(-1, -1, -1)  # lock resource
                 if ret:
-                    logger.info(f'allocated resource on {machine.hostname} for job {job.name} ')
+                    hostname = machine.hostname
+                    logger.info(f'allocated resource for {job.name},'
+                                f' data dir at: {hostname}{job.data_dir_path}')
                     executor = RemoteShellExecutor(job, self.api_server_portal, machine)
+
+                    job.set_ext({
+                        'backend_type': 'remote',
+                        'hostname': hostname
+                    })
                     # DOT NOT push anything to `_executors_map` or `_waiting_queue` if exception
                     self._executors_map[executor] = machine
-                    self._waiting_queue.append(executor)
+                    self._waiting_queue.append(executor)  # put it to queue if ready
                     return executor
         raise NoResourceException
 
@@ -337,11 +376,14 @@ class RemoteSSHExecutorManager(ExecutorManager):
         self.release_executor(executor)
 
     def release_executor(self, executor):
+        executor.close()  # close connection
+
         machine = self._executors_map.get(executor)
         logger.debug(f"release resource of machine {machine.hostname} ")
+
         machine.release(executor.job.resource)
         self._waiting_queue.remove(executor)
-        executor.close()
+
 
     def prepare(self):
         for machine in self.machines:
@@ -366,6 +408,7 @@ class LocalExecutorManager(ExecutorManager):
             self._allocated_executors.append(executor)
             self._waiting_queue.append(executor)
             self._is_busy = True
+            logger.info(f'allocated resource job {job.name}, data dir at: localhost{job.data_dir_path}')
             return executor
         raise NoResourceException
 
@@ -379,9 +422,11 @@ class LocalExecutorManager(ExecutorManager):
         executor.kill()
 
 
-def create_executor_manager(backend_type, backend_conf, server_host, server_port):  # instance factory
+def create_executor_manager(backend_conf, server_host, server_port):  # instance factory
     backend_conf = backend_conf if backend_conf is not None else {}
     server_portal = http_portal(server_host, server_port)
+    backend_type = backend_conf.get('type', 'local')
+
     if backend_type == 'remote':
         machines = [SSHRemoteMachine(**_) for _ in backend_conf['machines']]
         # check remote host setting, only warning for remote backend
