@@ -3,4 +3,4 @@
 
 """
 from ._primitives import CrossCategorical, GeoHashPrimitive, DaskCompatibleHaversine, TfidfPrimitive
-from ._transformers import FeatureGenerationTransformer
+from ._transformers import FeatureGenerationTransformer, is_geohash_installed
