@@ -1,3 +1,5 @@
+import time
+
 from sklearn.model_selection import train_test_split
 
 from hypernets.examples.plain_model import PlainModel, PlainSearchSpace
@@ -14,10 +16,14 @@ def main():
     experiment = make_experiment(PlainModel, df_train,
                                  target='target',
                                  search_space=search_space,
+                                 log_level='info',
+                                 random_state=8086,
                                  report_render='excel')
-    estimator = experiment.run(max_trials=3)
+    estimator = experiment.run(max_trials=10)
     print(estimator)
 
 
 if __name__ == '__main__':
+    t = time.time()
     main()
+    print(time.time() - t)
